<?php
/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link  https://webappick.com
 * @since 1.0.0
 *
 * @package    Woo_Invoice
 * @subpackage Woo_Invoice/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Woo_Invoice
 * @subpackage Woo_Invoice/includes
 * @author     Md Ohidul Islam <wahid@webappick.com>
 */
class Woo_Invoice {




	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since  1.0.0
	 * @access protected
	 * @var    Woo_Invoice_Loader $loader Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since  1.0.0
	 * @access protected
	 * @var    string $plugin_name The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since  1.0.0
	 * @access protected
	 * @var    string $version The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		if ( defined( 'WOO_INVOICE_FREE_VERSION' ) ) {
			$this->version = WOO_INVOICE_FREE_VERSION;
		} else {
			$this->version = '1.0.5';
		}

		// PDF SAVE DIRECTORY.
		$upload_dir      = wp_get_upload_dir();
		$base_dir        = $upload_dir['basedir'];
		$woo_invoice_dir = $upload_dir['basedir'] . '/WOO-INVOICE';
		if ( ! file_exists( $woo_invoice_dir ) && is_writable( $base_dir ) ) {
			mkdir( $woo_invoice_dir, 0777, true );
			define( 'WOO_INVOICE_DIR', $woo_invoice_dir . '/' );
		} else {
			if ( file_exists( $woo_invoice_dir ) && is_writable( $base_dir ) ) {
				define( 'WOO_INVOICE_DIR', $woo_invoice_dir . '/' );
			} else {
				define( 'WOO_INVOICE_DIR', $base_dir . '/' );
			}
		}

		$font_dir = $upload_dir['basedir'] . '/WOO-INVOICE-FONTS';
		if ( ! file_exists( $font_dir ) && is_writable( $base_dir ) ) {
			mkdir( $font_dir, 0777, true );
		}

		$this->plugin_name = 'woo-invoice';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();
		$this->protect_dir();
	}


	/**
	 * Generate htaccess file in woocommerce upload dir.
	 * Protect Woo Invoice File
	 */
	private function protect_dir() {
		// Protect invoice pdf directory.
		if ( ! file_exists( WOO_INVOICE_DIR . '.htaccess' ) ) {
			touch( WOO_INVOICE_DIR . '.htaccess' );
			$content = 'deny from all';
			$fp      = fopen( WOO_INVOICE_DIR . '.htaccess', 'wb' );
			fwrite( $fp, $content );
			fclose( $fp );
		}

		// Protect invoice font directory.
		if ( ! file_exists( WOO_INVOICE_FONT_DIR . '.htaccess' ) ) {
			touch( WOO_INVOICE_FONT_DIR . '.htaccess' );
			$content = 'deny from all';
			$fp      = fopen( WOO_INVOICE_FONT_DIR . '.htaccess', 'wb' );
			fwrite( $fp, $content );
			fclose( $fp );
		}

	}


	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Woo_Invoice_Loader. Orchestrates the hooks of the plugin.
	 * - Woo_Invoice_I18n. Defines internationalization functionality.
	 * - Woo_Invoice_Admin. Defines all hooks for the admin area.
	 * - Woo_Invoice_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since  1.0.0
	 * @access private
	 */
	private function load_dependencies() {
		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woo-invoice-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woo-invoice-i18n.php';

		/**
		 * The class responsible for defining PDF functionality
		 * of the plugin.
		 */
		include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woo-invoice-engine.php';

		/**
		 * The class responsible for loading PDF plugin hooks
		 * of the plugin.
		 */
		include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woo-invoice-hooks.php';

		/**
		 * The class responsible for initializing DOMPDF
		 * of the plugin.
		 */
		include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/vendor/autoload.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		include_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-woo-invoice-admin.php';

		include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woo-invoice-template.php';
		include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woo-invoice-helper.php';
		include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woo-invoice-orders.php';
		include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woo-invoice-pdf.php';
		include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/hooks.php';
		include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/widget.php';

		$this->loader = new Woo_Invoice_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Woo_Invoice_I18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since  1.0.0
	 * @access private
	 */
	private function set_locale() {
		$plugin_i18n = new Woo_Invoice_I18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since  1.0.0
	 * @access private
	 */
	private function define_admin_hooks() {
		$plugin_admin = new Woo_Invoice_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_base_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_common_scripts' );
		$this->loader->add_action( 'admin_menu', $plugin_admin, 'load_admin_menu' );

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since  1.0.0
	 * @access private
	 */
	private function define_public_hooks() {
		return false;
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since 1.0.0
	 */
	public function run() {
		 $this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since  1.0.0
	 * @return string    The name of the plugin.
	 */
	public function get_plugin_name() {
		 return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since  1.0.0
	 * @return Woo_Invoice_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since  1.0.0
	 * @return string    The version number of the plugin.
	 */
	public function get_version() {
		 return $this->version;
	}

}
